/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import {
  KommentarHistorieItemE2EComponent,
  PostfachNachrichtHistorieItemE2EComponent,
  VorgangFormularDatenHistorieItemE2EComponent,
  WiedervorlageHistorieItemE2EComponent,
} from 'apps/alfa-e2e/src/components/vorgang/vorgang.formular-daten.historie.e2e.component';
import { CommandE2E, CommandOrderE2E } from 'apps/alfa-e2e/src/model/command';
import {
  HistorieAttachmentE2E,
  HistorieHeadlineE2E,
  SYSTEM_USER_NAME,
} from 'apps/alfa-e2e/src/model/historie';
import { PostfachE2E } from 'apps/alfa-e2e/src/model/postfach-nachricht';
import { DirectionE2E } from 'apps/alfa-e2e/src/model/vorgang-attached-item';
import { buildCommand, createCommand, initCommands } from 'apps/alfa-e2e/src/support/command-util';
import { createKommentar } from 'apps/alfa-e2e/src/support/kommentar.util';
import { createPostfachNachricht } from 'apps/alfa-e2e/src/support/postfach-util';
import { createWiedervorlageItem } from 'apps/alfa-e2e/src/support/wiedervorlage-util';
import { VorgangDetailHeaderE2EComponent } from '../../../components/vorgang/vorgang-detail-header.e2e.component';
import { VorgangFormularDatenE2EComponent } from '../../../components/vorgang/vorgang-formular.e2e.component';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { VorgangE2E } from '../../../model/vorgang';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { dropCollections } from '../../../support/cypress-helper';
import { contains, exist, notExist } from '../../../support/cypress.util';
import {
  getUserSabine,
  getUserSabineId,
  initUsermanagerUsers,
  loginAsSabine,
} from '../../../support/user-util';
import { createVorgang, initVorgang } from '../../../support/vorgang-util';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('Historie', () => {
  const mainPage: MainPage = new MainPage();
  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

  const vorgangPage: VorgangPage = new VorgangPage();
  const vorgangDatenFormular: VorgangFormularDatenE2EComponent =
    vorgangPage.getFormularDatenContainer();

  const vorgangHeader: VorgangDetailHeaderE2EComponent = vorgangPage.getVorgangDetailHeader();

  const vorgang: VorgangE2E = createVorgang();

  const assignUserCommand: CommandE2E = {
    ...buildCommand(CommandOrderE2E.ASSIGN_USER, vorgang._id.$oid, vorgang._id.$oid),
    bodyObject: { assignedTo: getUserSabineId() },
    createdAt: { $date: '2020-01-01T10:00:00.000Z' },
  };

  const createKommentarCommandBody = {
    itemName: 'Kommentar',
    item: { ...createKommentar(), attachments: 'DummyAttachment' },
  };
  const createKommentarCommand: CommandE2E = {
    ...buildCommand(CommandOrderE2E.CREATE_ATTACHED_ITEM, vorgang._id.$oid, vorgang._id.$oid),
    bodyObject: createKommentarCommandBody,
    createdAt: { $date: '2020-01-01T10:01:00.000Z' },
  };
  const editKommentarCommandBody = {
    itemName: 'Kommentar',
    item: { ...createKommentar(), attachments: ['DummyAttachment', 'DummyAttachment2'] },
  };
  const editKommentarCommand: CommandE2E = {
    ...buildCommand(CommandOrderE2E.UPDATE_ATTACHED_ITEM, vorgang._id.$oid, vorgang._id.$oid),
    bodyObject: editKommentarCommandBody,
    createdAt: { $date: '2020-01-01T10:02:00.000Z' },
  };

  const forwardVorgangCommand: CommandE2E = {
    ...buildCommand(CommandOrderE2E.FORWARD_VORGANG, vorgang._id.$oid, vorgang._id.$oid),
    createdAt: { $date: '2020-01-01T10:03:00.000Z' },
  };
  const forwardSuccessfulCommand: CommandE2E = {
    ...buildCommand(CommandOrderE2E.FORWARD_SUCCESSFUL, vorgang._id.$oid, vorgang._id.$oid),
    createdAt: { $date: '2020-01-01T10:04:00.000Z' },
  };
  const forwardFailedCommand: CommandE2E = {
    ...buildCommand(CommandOrderE2E.FORWARD_FAILED, vorgang._id.$oid, vorgang._id.$oid),
    createdAt: { $date: '2020-01-01T10:05:00.000Z' },
  };

  const sendPostfachMailCommandBody: PostfachE2E = {
    ...createPostfachNachricht(),
    mailBody: 'With mail test body!',
    subject: 'Send Postfach Mail',
    attachments: 'dummyAttachment',
  };
  const sendPostfachMailCommand: CommandE2E = {
    ...createCommand(),
    bodyObject: sendPostfachMailCommandBody,
    createdAt: { $date: '2020-01-01T10:06:00.000Z' },
  };
  const sendPostfachNachrichtCommandBody: PostfachE2E = {
    ...createPostfachNachricht(),
    mailBody: 'With nachricht test body!',
    subject: 'Send Postfach Nachricht',
    attachments: ['dummyAttachment', 'dummyAttachment2'],
  };
  const sendPostfachNachrichtCommand: CommandE2E = {
    ...buildCommand(CommandOrderE2E.SEND_POSTFACH_NACHRICHT, vorgang._id.$oid, vorgang._id.$oid),
    bodyObject: sendPostfachNachrichtCommandBody,
    createdAt: { $date: '2020-01-01T10:07:00.000Z' },
  };
  const receivePostfachNachrichtCommand: CommandE2E = {
    ...buildCommand(CommandOrderE2E.CREATE_ATTACHED_ITEM, vorgang._id.$oid, vorgang._id.$oid),
    bodyObject: {
      itemName: 'PostfachMail',
      item: {
        mailBody: 'With nachricht test body!',
        attachments: ['dummyAttachment', 'dummyAttachment2'],
        subject: 'AW: Send Postfach Nachricht',
        direction: DirectionE2E.IN,
      },
      client: 'OzgCloud_NachrichtenManager',
      vorgangId: vorgang._id.$oid,
    },
    createdBy: null,
    createdAt: { $date: '2020-01-01T10:08:00.000Z' },
  };

  const resendPostfachNachricht: CommandE2E = buildCommand(
    CommandOrderE2E.RESEND_POSTFACH_NACHRICHT,
    vorgang._id.$oid,
    vorgang._id.$oid,
  );

  const createWiedervorlageCommandBody = {
    ...createWiedervorlageItem('Create Wiedervorlage Betreff'),
    attachments: 'DummyAttachment',
  };
  const createWiedervorlageCommand: CommandE2E = {
    ...buildCommand(CommandOrderE2E.CREATE_ATTACHED_ITEM, vorgang._id.$oid, vorgang._id.$oid),
    bodyObject: { itemName: 'Wiedervorlage', item: createWiedervorlageCommandBody },
    createdAt: { $date: '2020-01-01T10:09:00.000Z' },
  };
  const editWiedervorlageCommandBody = {
    ...createWiedervorlageItem('Edit Wiedervorlage Betreff'),
    attachments: ['DummyAttachment', 'DummyAttachment2'],
    done: true,
  };
  const editWiedervorlageCommand: CommandE2E = {
    ...buildCommand(CommandOrderE2E.UPDATE_ATTACHED_ITEM, vorgang._id.$oid, vorgang._id.$oid),
    bodyObject: { itemName: 'Wiedervorlage', item: editWiedervorlageCommandBody },
    createdAt: { $date: '2020-01-01T10:10:00.000Z' },
  };
  const wiedervorlageErledigenCommand: CommandE2E = {
    ...buildCommand(CommandOrderE2E.PATCH_ATTACHED_ITEM, vorgang._id.$oid, vorgang._id.$oid),
    bodyObject: { itemName: 'Wiedervorlage', item: { done: 'true' }, vorgangId: vorgang._id.$oid },
    createdAt: { $date: '2020-01-01T10:11:00.000Z' },
  };
  const wiedervorlageWiedereroeffnenCommand: CommandE2E = {
    ...buildCommand(CommandOrderE2E.PATCH_ATTACHED_ITEM, vorgang._id.$oid, vorgang._id.$oid),
    bodyObject: { itemName: 'Wiedervorlage', item: { done: 'false' }, vorgangId: vorgang._id.$oid },
    createdAt: { $date: '2020-01-01T10:12:00.000Z' },
  };

  before(() => {
    initVorgang(vorgang);
    initCommands([
      assignUserCommand,
      createKommentarCommand,
      editKommentarCommand,
      forwardVorgangCommand,
      forwardSuccessfulCommand,
      forwardFailedCommand,
      sendPostfachMailCommand,
      sendPostfachNachrichtCommand,
      receivePostfachNachrichtCommand,
      resendPostfachNachricht,
      createWiedervorlageCommand,
      editWiedervorlageCommand,
      wiedervorlageErledigenCommand,
      wiedervorlageWiedereroeffnenCommand,
    ]);
    initUsermanagerUsers();

    loginAsSabine();

    waitForSpinnerToDisappear();
    exist(vorgangList.getRoot());
  });

  after(() => {
    dropCollections();
  });

  describe('navigate to vorgang detail', () => {
    describe('click on vorgang in list', () => {
      it('should show header', () => {
        vorgangList.getListItem(vorgang.name).getRoot().click();
        waitForSpinnerToDisappear();

        exist(vorgangHeader.getRoot());
      });
    });

    describe('historie in tab', () => {
      it('should show on click on tab', () => {
        vorgangDatenFormular.getHistorieTab().click();

        exist(vorgangDatenFormular.getHistorieContainer().getRoot());
      });

      it('should show vorgang created at', () => {
        exist(vorgangDatenFormular.getHistorieContainer().getCreatedAt());
      });

      describe('should show historie item for', () => {
        const userName: string = `${getUserSabine().firstName} ${getUserSabine().lastName}`;

        describe('vorgang order', () => {
          it('assign user', () => {
            const historieItem: VorgangFormularDatenHistorieItemE2EComponent = vorgangDatenFormular
              .getHistorieContainer()
              .getListItemByIndex(0);

            exist(historieItem.getRoot());
            contains(historieItem.getHeadline(), HistorieHeadlineE2E.ASSIGN_USER);
            contains(historieItem.getUser(), userName);
            notExist(historieItem.getExpandButton());
          });
        });

        describe('kommentar order', () => {
          it('create kommentar', () => {
            const historieItem: VorgangFormularDatenHistorieItemE2EComponent = vorgangDatenFormular
              .getHistorieContainer()
              .getListItemByIndex(1);

            exist(historieItem.getRoot());
            contains(historieItem.getHeadline(), HistorieHeadlineE2E.CREATE_KOMMENTAR);
            contains(historieItem.getUser(), userName);

            historieItem.getExpandButton().click();
            const kommentarItem: KommentarHistorieItemE2EComponent = historieItem.getKommentar();

            contains(kommentarItem.getText(), createKommentarCommandBody.item.text);
            contains(kommentarItem.getAttachment(), HistorieAttachmentE2E.SINGLE_TEXT);
          });

          it('edit kommentar', () => {
            const historieItem: VorgangFormularDatenHistorieItemE2EComponent = vorgangDatenFormular
              .getHistorieContainer()
              .getListItemByIndex(2);

            exist(historieItem.getRoot());
            contains(historieItem.getHeadline(), HistorieHeadlineE2E.EDIT_KOMMENTAR);
            contains(historieItem.getUser(), userName);

            historieItem.getExpandButton().click();
            const kommentarItem: KommentarHistorieItemE2EComponent = historieItem.getKommentar();

            contains(kommentarItem.getText(), editKommentarCommandBody.item.text);
            contains(
              kommentarItem.getAttachment(),
              HistorieAttachmentE2E.MULTIPLE_TEXT.replace('{countAttachments}', '2'),
            );
          });
        });

        describe('forward order', () => {
          it('forward vorgang', () => {
            const historieItem: VorgangFormularDatenHistorieItemE2EComponent = vorgangDatenFormular
              .getHistorieContainer()
              .getListItemByIndex(3);

            exist(historieItem.getRoot());
            contains(historieItem.getHeadline(), HistorieHeadlineE2E.FORWARD_VORGANG);
            contains(historieItem.getUser(), userName);
            notExist(historieItem.getExpandButton());
          });

          it('foward successful', () => {
            const historieItem: VorgangFormularDatenHistorieItemE2EComponent = vorgangDatenFormular
              .getHistorieContainer()
              .getListItemByIndex(4);

            exist(historieItem.getRoot());
            contains(historieItem.getHeadline(), HistorieHeadlineE2E.FORWARD_SUCCESSFUL);
            contains(historieItem.getUser(), userName);
            notExist(historieItem.getExpandButton());
          });

          it('foward failed', () => {
            const historieItem: VorgangFormularDatenHistorieItemE2EComponent = vorgangDatenFormular
              .getHistorieContainer()
              .getListItemByIndex(5);

            exist(historieItem.getRoot());
            contains(historieItem.getHeadline(), HistorieHeadlineE2E.FORWARD_FAILED);
            contains(historieItem.getUser(), userName);
            notExist(historieItem.getExpandButton());
          });
        });

        describe('postfach nachricht order', () => {
          it('send postfach mail', () => {
            const historieItem: VorgangFormularDatenHistorieItemE2EComponent = vorgangDatenFormular
              .getHistorieContainer()
              .getListItemByIndex(6);

            exist(historieItem.getRoot());
            contains(historieItem.getHeadline(), HistorieHeadlineE2E.SEND_POSTFACH_NACHRICHT);
            contains(historieItem.getUser(), userName);

            historieItem.getExpandButton().click();
            const postfachNachtichtItem: PostfachNachrichtHistorieItemE2EComponent =
              historieItem.getPostfachNachricht();

            contains(
              postfachNachtichtItem.getPostfachNachrichtSubject(),
              sendPostfachMailCommandBody.subject,
            );
            contains(
              postfachNachtichtItem.getPostfachNachrichtMailBody(),
              sendPostfachMailCommandBody.mailBody,
            );
          });

          it('send postfach nachricht', () => {
            const historieItem: VorgangFormularDatenHistorieItemE2EComponent = vorgangDatenFormular
              .getHistorieContainer()
              .getListItemByIndex(7);

            exist(historieItem.getRoot());
            contains(historieItem.getHeadline(), HistorieHeadlineE2E.SEND_POSTFACH_NACHRICHT);
            contains(historieItem.getUser(), userName);

            historieItem.getExpandButton().click();
            const postfachNachtichtItem: PostfachNachrichtHistorieItemE2EComponent =
              historieItem.getPostfachNachricht();

            contains(
              postfachNachtichtItem.getPostfachNachrichtSubject(),
              sendPostfachNachrichtCommandBody.subject,
            );
            contains(
              postfachNachtichtItem.getPostfachNachrichtMailBody(),
              sendPostfachNachrichtCommandBody.mailBody,
            );
          });

          it('receive postfach nachricht', () => {
            const historieItem: VorgangFormularDatenHistorieItemE2EComponent = vorgangDatenFormular
              .getHistorieContainer()
              .getListItemByIndex(8);

            exist(historieItem.getRoot());
            contains(historieItem.getHeadline(), HistorieHeadlineE2E.RECEIVE_POSTFACH_NACHRICHT);
            notExist(historieItem.getUser());
            contains(historieItem.getSystemUser(), SYSTEM_USER_NAME);

            historieItem.getExpandButton().click();
            const postfachNachtichtItem: PostfachNachrichtHistorieItemE2EComponent =
              historieItem.getPostfachNachricht();

            contains(
              postfachNachtichtItem.getPostfachNachrichtSubject(),
              receivePostfachNachrichtCommand.bodyObject.item.subject,
            );
            contains(
              postfachNachtichtItem.getPostfachNachrichtMailBody(),
              receivePostfachNachrichtCommand.bodyObject.item.mailBody,
            );
          });

          it('resend postfach nachricht', () => {
            const historieItem: VorgangFormularDatenHistorieItemE2EComponent = vorgangDatenFormular
              .getHistorieContainer()
              .getListItemByIndex(9);

            exist(historieItem.getRoot());
            contains(historieItem.getHeadline(), HistorieHeadlineE2E.RESEND_POSTFACH_NACHRICHT);
            contains(historieItem.getUser(), userName);
          });
        });

        describe('wiedervorlage order', () => {
          it('create wiedervorlage', () => {
            const historieItem: VorgangFormularDatenHistorieItemE2EComponent = vorgangDatenFormular
              .getHistorieContainer()
              .getListItemByIndex(10);

            exist(historieItem.getRoot());
            contains(historieItem.getHeadline(), HistorieHeadlineE2E.CREATE_WIEDERVORLAGE);
            contains(historieItem.getUser(), userName);

            historieItem.getExpandButton().click();
            const wiedervorlageItem: WiedervorlageHistorieItemE2EComponent =
              historieItem.getWiedervorlage();

            contains(wiedervorlageItem.getStatus(), 'offen');
            contains(wiedervorlageItem.getBetreff(), createWiedervorlageCommandBody.betreff);
            contains(
              wiedervorlageItem.getBeschreibung(),
              createWiedervorlageCommandBody.beschreibung,
            );
            contains(wiedervorlageItem.getAttachment(), HistorieAttachmentE2E.SINGLE_TEXT);
          });

          it('edit wiedervorlage', () => {
            const historieItem: VorgangFormularDatenHistorieItemE2EComponent = vorgangDatenFormular
              .getHistorieContainer()
              .getListItemByIndex(11);

            exist(historieItem.getRoot());
            contains(historieItem.getHeadline(), HistorieHeadlineE2E.EDIT_WIEDERVORLAGE);
            contains(historieItem.getUser(), userName);

            historieItem.getExpandButton().click();
            const wiedervorlageItem: WiedervorlageHistorieItemE2EComponent =
              historieItem.getWiedervorlage();

            contains(wiedervorlageItem.getStatus(), 'geschlossen');
            contains(wiedervorlageItem.getBetreff(), editWiedervorlageCommandBody.betreff);
            contains(
              wiedervorlageItem.getBeschreibung(),
              editWiedervorlageCommandBody.beschreibung,
            );
            contains(
              wiedervorlageItem.getAttachment(),
              HistorieAttachmentE2E.MULTIPLE_TEXT.replace('{countAttachments}', '2'),
            );
          });

          it('wiedervorlage erledigen', () => {
            const historieItem: VorgangFormularDatenHistorieItemE2EComponent = vorgangDatenFormular
              .getHistorieContainer()
              .getListItemByIndex(12);

            exist(historieItem.getRoot());
            contains(historieItem.getHeadline(), HistorieHeadlineE2E.WIEDERVORLAGE_ERLEDIGEN);
            contains(historieItem.getUser(), userName);
            notExist(historieItem.getExpandButton());
          });

          it('wiedervorlage wiedereroeffnen', () => {
            const historieItem: VorgangFormularDatenHistorieItemE2EComponent = vorgangDatenFormular
              .getHistorieContainer()
              .getListItemByIndex(13);

            exist(historieItem.getRoot());
            contains(historieItem.getHeadline(), HistorieHeadlineE2E.WIEDERVORLAGE_WIEDEREROEFFNEN);
            contains(historieItem.getUser(), userName);
            notExist(historieItem.getExpandButton());
          });
        });
      });
    });
  });
});
