/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { VorgangAttachedItemE2E } from 'apps/alfa-e2e/src/model/vorgang-attached-item';
import {
  createDateInTheFuture,
  createDateInThePast,
  createDateToday,
} from 'apps/alfa-e2e/src/support/tech.util';
import { initVorgangAttachedItem } from 'apps/alfa-e2e/src/support/vorgang-attached-item-util';
import {
  createWiedervorlageAttachedItem,
  createWiedervorlageItem,
} from 'apps/alfa-e2e/src/support/wiedervorlage-util';
import { VorgangListItemE2EComponent } from '../../../components/vorgang/vorgang-item.e2e.component';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { ClientAttributeNameE2E, VorgangE2E } from '../../../model/vorgang';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { dropCollections } from '../../../support/cypress-helper';
import { containClass, exist, notContainClass, notExist } from '../../../support/cypress.util';
import { loginAsSabine } from '../../../support/user-util';
import {
  buildVorgang,
  createAlfaClientAttributes,
  createNextWiedervorlageFristClientAttribute,
  createVorgang,
  initVorgaenge,
  objectIds,
} from '../../../support/vorgang-util';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('VorgangList Wiedervorlagen Next Frist', () => {
  const mainPage: MainPage = new MainPage();
  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

  const vorgangWithNoWiedervorlagen: VorgangE2E = { ...createVorgang(), name: 'NoWiedervorlagen' };

  const vorgangWithWiedervorlageFristToday: VorgangE2E = {
    ...buildVorgang(objectIds[0], 'FristToday'),
    clientAttributes: createAlfaClientAttributes(
      ClientAttributeNameE2E.NEXT_WIEDERVORLAGE_FRIST,
      createNextWiedervorlageFristClientAttribute(createDateToday()),
    ),
  };
  const wiedervorlageFristToday: VorgangAttachedItemE2E = {
    ...createWiedervorlageAttachedItem(objectIds[1], vorgangWithWiedervorlageFristToday._id.$oid),
    item: { ...createWiedervorlageItem('WiedervorlageFristToday'), frist: createDateToday() },
  };

  const vorgangWithWiedervorlageInThePast: VorgangE2E = {
    ...buildVorgang(objectIds[2], 'FristInThePast'),
    clientAttributes: createAlfaClientAttributes(
      ClientAttributeNameE2E.NEXT_WIEDERVORLAGE_FRIST,
      createNextWiedervorlageFristClientAttribute(createDateInThePast()),
    ),
  };
  const wiedervorlageFristInThePast: VorgangAttachedItemE2E = {
    ...createWiedervorlageAttachedItem(objectIds[3], vorgangWithWiedervorlageInThePast._id.$oid),
    item: {
      ...createWiedervorlageItem('WiedervorlageFristInThePast'),
      frist: createDateInThePast(),
    },
  };

  const vorgangWithWiedervorlageInFuture: VorgangE2E = {
    ...buildVorgang(objectIds[4], 'FristInTheFuture'),
    clientAttributes: createAlfaClientAttributes(
      ClientAttributeNameE2E.NEXT_WIEDERVORLAGE_FRIST,
      createNextWiedervorlageFristClientAttribute(createDateInTheFuture()),
    ),
  };
  const wiedervorlageFristInTheFuture: VorgangAttachedItemE2E = {
    ...createWiedervorlageAttachedItem(objectIds[5], vorgangWithWiedervorlageInFuture._id.$oid),
    item: {
      ...createWiedervorlageItem('WiedervorlageFristInTheFuture'),
      frist: createDateInTheFuture(),
    },
  };

  before(() => {
    initVorgaenge([
      vorgangWithNoWiedervorlagen,
      vorgangWithWiedervorlageFristToday,
      vorgangWithWiedervorlageInThePast,
      vorgangWithWiedervorlageInFuture,
    ]);

    initVorgangAttachedItem([
      wiedervorlageFristToday,
      wiedervorlageFristInThePast,
      wiedervorlageFristInTheFuture,
    ]);

    loginAsSabine();

    waitForSpinnerToDisappear();
    exist(vorgangList.getRoot());
  });

  after(() => {
    dropCollections();
  });

  describe('on vorgang with no wiedervorlagen', () => {
    it('should not show next frist icon', () => {
      const item: VorgangListItemE2EComponent = vorgangList.getListItem(
        vorgangWithNoWiedervorlagen.name,
      );

      notExist(item.getWiedervorlageNextFrist());
    });
  });

  describe('on vorgang with existing wiedervorlagen', () => {
    it('should show next frist in future and list', () => {
      const vorgang: VorgangListItemE2EComponent = vorgangList.getListItem(
        vorgangWithWiedervorlageInFuture.name,
      );

      notContainClass(vorgang.getWiedervorlageNextFrist(), 'red');
      exist(vorgang.getWiedervorlageNextFrist());
    });

    it('should show next frist in the past and list', () => {
      const vorgang: VorgangListItemE2EComponent = vorgangList.getListItem(
        vorgangWithWiedervorlageInThePast.name,
      );

      containClass(vorgang.getWiedervorlageNextFrist(), 'red');
      exist(vorgang.getWiedervorlageNextFrist());
    });
  });
});
