import { BescheidLinkRel, BescheidListResource } from '@alfa-client/bescheid-shared';
import { BinaryFileListContainerComponent } from '@alfa-client/binary-file';
import { GetUrlPipe, HasLinkPipe, ToEmbeddedResourcesPipe } from '@alfa-client/tech-shared';
import { existsAsHtmlElement, notExistsAsHtmlElement } from '@alfa-client/test-utils';
import { DatePipe } from '@angular/common';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatIcon } from '@angular/material/icon';
import { BescheidContainerComponent, BescheidStatusTextComponent } from '@ods/system';
import {
  createBescheidListResource,
  createBescheidResource,
} from 'libs/bescheid-shared/src/test/bescheid';
import { getDataTestClassOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { BescheidListInVorgangComponent } from './bescheid-list-in-vorgang.component';
import { DocumentInBescheidContainerComponent } from './document-in-bescheid-container/document-in-bescheid-container.component';

describe('BescheidListInVorgangComponent', () => {
  let component: BescheidListInVorgangComponent;
  let fixture: ComponentFixture<BescheidListInVorgangComponent>;

  const documentInBescheidContainer: string = getDataTestClassOf('document-in-bescheid-container');
  const binaryFileListContainer: string = getDataTestClassOf('binary-file-container-in-bescheid');
  const bescheidStatusText: string = getDataTestClassOf('bescheid-status-text');

  const bescheidListResource: BescheidListResource = createBescheidListResource();

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        BescheidListInVorgangComponent,
        ToEmbeddedResourcesPipe,
        DatePipe,
        GetUrlPipe,
        HasLinkPipe,
        MatIcon,
        MockComponent(DocumentInBescheidContainerComponent),
        MockComponent(BinaryFileListContainerComponent),
        MockComponent(BescheidStatusTextComponent),
        MockComponent(BescheidContainerComponent),
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(BescheidListInVorgangComponent);
    component = fixture.componentInstance;
    component.bescheidList = bescheidListResource;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('status text', () => {
    it('should show', () => {
      fixture.detectChanges();

      existsAsHtmlElement(fixture, bescheidStatusText);
    });
  });

  describe('document', () => {
    it('should be visible if link is present', () => {
      component.bescheidList = createBescheidListResource([
        createBescheidResource([BescheidLinkRel.BESCHEID_DOCUMENT]),
      ]);

      fixture.detectChanges();

      existsAsHtmlElement(fixture, documentInBescheidContainer);
    });

    it('should be hidden if link is not present', () => {
      component.bescheidList = createBescheidListResource([createBescheidResource()]);

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, documentInBescheidContainer);
    });
  });

  describe('attachments', () => {
    it('should be visible if link is present', () => {
      component.bescheidList = createBescheidListResource([
        createBescheidResource([BescheidLinkRel.ATTACHMENTS]),
      ]);

      fixture.detectChanges();

      existsAsHtmlElement(fixture, binaryFileListContainer);
    });

    it('should be hidden if link is not present', () => {
      component.bescheidList = createBescheidListResource([createBescheidResource()]);

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, binaryFileListContainer);
    });
  });
});
