import { BinaryFileResource, BinaryFileService } from '@alfa-client/binary-file-shared';
import { LinkRelationName, StateResource, createStateResource } from '@alfa-client/tech-shared';
import { Mock, getMockComponent, mock } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { Resource } from '@ngxp/rest';
import { createBinaryFileResource } from 'libs/binary-file-shared/test/binary-file';
import { DummyLinkRel } from 'libs/tech-shared/test/dummy';
import { createDummyResource } from 'libs/tech-shared/test/resource';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { BinaryFileListContainerComponent } from './binary-file-list-container.component';
import { BinaryFileListComponent } from './binary-file-list/binary-file-list.component';

import { AttachmentContainerComponent } from '@ods/system';

describe('BinaryFileListContainerComponent', () => {
  let component: BinaryFileListContainerComponent;
  let fixture: ComponentFixture<BinaryFileListContainerComponent>;

  const binaryFileService: Mock<BinaryFileService> = mock(BinaryFileService);

  const binaryFileStateResource: StateResource<BinaryFileResource> = createStateResource(
    createBinaryFileResource(),
  );
  const resource: Resource = createDummyResource();
  const linkRel: LinkRelationName = DummyLinkRel.DUMMY;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        BinaryFileListContainerComponent,
        MockComponent(BinaryFileListComponent),
        MockComponent(AttachmentContainerComponent),
      ],
      providers: [
        {
          provide: BinaryFileService,
          useValue: binaryFileService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(BinaryFileListContainerComponent);
    component = fixture.componentInstance;
    component.resource = resource;
    component.linkRel = linkRel;
    binaryFileService.getFiles.mockReturnValue(of(binaryFileStateResource));
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnInit', () => {
    it('should call service to get files', () => {
      component.ngOnInit();

      expect(binaryFileService.getFiles).toHaveBeenCalledWith(resource, linkRel);
    });
  });

  describe('binary file list', () => {
    it('should be called with binary file state resource', () => {
      const binaryFileListComponent: BinaryFileListComponent =
        getMockComponent<BinaryFileListComponent>(fixture, BinaryFileListComponent);

      expect(binaryFileListComponent.binaryFileListStateResource).toBe(binaryFileStateResource);
    });
  });
});
