/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { faker } from '@faker-js/faker';
import { toResource } from 'libs/tech-shared/test/resource';
import { times } from 'lodash-es';
import { CommandListLinkRel } from '../src/lib/command.linkrel';
import { CommandErrorMessage } from '../src/lib/command.message';
import {
  Command,
  CommandListResource,
  CommandOrder,
  CommandResource,
  CommandStatus,
  CreateCommand,
  CreateCommandProps,
} from '../src/lib/command.model';
import { createStateResource, StateResource } from '@alfa-client/tech-shared';

export function createCommand(): Command {
  return {
    body: null,
    createdAt: faker.date.past(),
    finishedAt: faker.date.past(),
    order: faker.random.word(),
    status: CommandStatus.FINISHED,
    errorMessage: null,
  };
}

export function createCommandResource(linkRel: string[] = []): CommandResource {
  return toResource(createCommand(), linkRel);
}

export function createCommandResources(linkRelations: string[] = []): CommandResource[] {
  return times(10, () => toResource(createCommandResource(), [...linkRelations]));
}

export function createCommandStateResource(linkRel: string[] = []): StateResource<CommandResource> {
  return createStateResource(createCommandResource(linkRel));
}

export function createCommandListResource(
  commandResources: CommandResource[] = createCommandResources(),
  linkRelations: string[] = [],
): CommandListResource {
  return toResource({}, [...linkRelations], {
    [CommandListLinkRel.COMMAND_LIST]: commandResources,
  });
}

export function createCommandErrorResource(linkRelations: string[] = []): CommandResource {
  return {
    ...createCommandResource(linkRelations),
    errorMessage: CommandErrorMessage.CONCURRENT_MODIFICATION,
  };
}

export function createCreateCommand(
  order: CommandOrder = CommandOrder.VORGANG_ANNEHMEN,
): CreateCommand {
  return { order, body: null };
}

export function createCreateCommandProps(): CreateCommandProps {
  return {
    resource: createCommandResource(),
    linkRel: faker.internet.url(),
    command: createCreateCommand(),
    snackBarMessage: faker.random.words(5),
  };
}
