import { StateResource, TechSharedModule } from '@alfa-client/tech-shared';
import { FormControlEditorAbstractComponent } from '@alfa-client/ui';
import { NgForOf } from '@angular/common';
import { Component, EventEmitter, HostListener, Input, OnInit, Output } from '@angular/core';
import {
  ControlContainer,
  FormGroupDirective,
  ReactiveFormsModule,
  UntypedFormArray,
  UntypedFormControl,
} from '@angular/forms';
import { Resource } from '@ngxp/rest';
import {
  AttachmentIconComponent,
  FileUploadButtonComponent,
  SpinnerIconComponent,
} from '@ods/system';
import { uniqueId } from 'lodash-es';

@Component({
  selector: 'ods-file-upload-editor',
  templateUrl: './file-upload-editor.component.html',
  viewProviders: [{ provide: ControlContainer, useExisting: FormGroupDirective }],
  standalone: true,
  imports: [
    FileUploadButtonComponent,
    AttachmentIconComponent,
    SpinnerIconComponent,
    ReactiveFormsModule,
    NgForOf,
    TechSharedModule,
  ],
})
export class FileUploadEditorComponent
  extends FormControlEditorAbstractComponent
  implements OnInit
{
  @Input() label: string = '';
  @Input() parentFormArrayName: string;
  @Input() accept: string = '*/*';
  @Input() uploadInProgress: StateResource<Resource>;
  @Input() set fileLinkList(value: string[]) {
    this.buildFormArray(value);
  }

  @Output() public newFile: EventEmitter<File> = new EventEmitter<File>();

  fileLinkControls: UntypedFormArray = new UntypedFormArray([]);

  readonly uploadFileId: string = uniqueId();

  @HostListener('change', ['$event.target.files']) emitFiles(event: FileList): void {
    const file = event && event.item(0);
    this.upload(file);
    this.setErrors();
  }

  constructor(public parentForm: FormGroupDirective) {
    super(null);
  }

  override ngOnInit(): void {
    this.fileLinkControls = this.parentForm.form.get(this.parentFormArrayName) as UntypedFormArray;
  }

  buildFormArray(fileLinkList: string[]): void {
    this.fileLinkControls.clear();
    fileLinkList.forEach((link: string) =>
      this.fileLinkControls.push(new UntypedFormControl(link)),
    );
  }

  upload(file: File): void {
    this.newFile.emit(file);
  }
}
