import { getElementFromFixture } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ReactiveFormsModule } from '@angular/forms';
import faker from '@faker-js/faker';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { ValidationErrorComponent } from '../validation-error/validation-error.component';
import { TextEditorComponent } from './text-editor.component';

describe('TextEditorComponent', () => {
  let component: TextEditorComponent;
  let fixture: ComponentFixture<TextEditorComponent>;

  const labelText: string = faker.word.noun();
  const inputTestId: string = getDataTestIdOf(labelText + '-text-editor');
  const errorId: string = getDataTestIdOf(labelText + '-text-editor-error');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [TextEditorComponent, MockComponent(ValidationErrorComponent)],
      imports: [TextEditorComponent, ReactiveFormsModule],
    }).compileComponents();

    fixture = TestBed.createComponent(TextEditorComponent);
    component = fixture.componentInstance;
    component.label = labelText;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('should have text editor', () => {
    const element: HTMLElement = getElementFromFixture(fixture, inputTestId);

    expect(element).toBeInstanceOf(HTMLElement);
  });

  describe('errors', () => {
    it('should be hidden', () => {
      const element: HTMLElement = getElementFromFixture(fixture, errorId);

      expect(element).toBeEmptyDOMElement();
    });

    // TODO: Set error and test for 'should be visible
  });
});
