import { CommonModule } from '@angular/common';
import { Component, Input } from '@angular/core';

import { FileIconComponent } from '../icons/file-icon/file-icon.component';
import { SpinnerIconComponent } from '../icons/spinner-icon/spinner-icon.component';

@Component({
  selector: 'ods-attachment',
  standalone: true,
  imports: [CommonModule, SpinnerIconComponent, FileIconComponent],
  styles: [':host {@apply flex}'],
  template: `<button
    class="border-b-grayborder relative flex w-full items-start gap-3 border-b bg-background-100 px-3 py-2 hover:bg-background-200"
  >
    <div class="flex-shrink">
      <ods-file-icon *ngIf="!isLoading" [fileType]="fileType" size="large" />
      <ods-spinner-icon *ngIf="isLoading" size="large" />
    </div>
    <div class="flex grow flex-col items-start break-all text-text">
      <p class="text-start text-sm">
        {{ documentName }}
      </p>
      <p class="text-xs text-text/65">{{ description }}</p>
    </div>
    <ng-content select="[close]" *ngIf="!isLoading"></ng-content>
  </button>`,
})
export class AttachmentComponent {
  @Input({ required: true }) documentName!: string;
  @Input({ required: true }) fileType!: string;
  @Input() description = '';
  @Input() isLoading: boolean = false;
}
