/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { KommentarResource, KommentarService } from '@alfa-client/kommentar-shared';
import { AbstractFormService, StateResource } from '@alfa-client/tech-shared';
import { Injectable } from '@angular/core';
import {
  UntypedFormArray,
  UntypedFormBuilder,
  UntypedFormControl,
  UntypedFormGroup,
} from '@angular/forms';
import { Observable } from 'rxjs';

@Injectable()
export class KommentarFormService extends AbstractFormService<KommentarResource> {
  static readonly TEXT = 'text';

  static readonly FIELD_PATH_PREFIX = 'kommentar';
  static readonly FIELD_ATTACHMENTS = 'attachments';

  constructor(
    formBuilder: UntypedFormBuilder,
    private kommentarService: KommentarService,
  ) {
    super(formBuilder);
  }

  protected initForm(): UntypedFormGroup {
    return this.formBuilder.group({
      [KommentarFormService.TEXT]: new UntypedFormControl(null),
      [KommentarFormService.FIELD_ATTACHMENTS]: new UntypedFormArray([]),
    });
  }

  protected doSubmit(): Observable<StateResource<any>> {
    if (this.isPatch()) {
      return this.kommentarService.editKommentar(this.getSourceValue(), this.getFormValue());
    } else {
      return this.kommentarService.createKommentar(this.getFormValue());
    }
  }

  protected getPathPrefix(): string {
    return KommentarFormService.FIELD_PATH_PREFIX;
  }
}
