/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { CommandResource } from '@alfa-client/command-shared';
import { PostfachService } from '@alfa-client/postfach-shared';
import { AbstractFormService, StateResource } from '@alfa-client/tech-shared';
import { Injectable } from '@angular/core';
import {
  UntypedFormArray,
  UntypedFormBuilder,
  UntypedFormControl,
  UntypedFormGroup,
} from '@angular/forms';
import { ReplyOption } from 'libs/postfach-shared/src/lib/postfach.model';
import { Observable } from 'rxjs';

@Injectable()
export class PostfachMailFormservice extends AbstractFormService<CommandResource> {
  static readonly FIELD_PATH_PREFIX = 'command.body';

  static readonly FIELD_SUBJECT = 'subject';
  static readonly FIELD_MAIL_BODY = 'mailBody';
  static readonly FIELD_REPLY_OPTION = 'replyOption';
  static readonly FIELD_ATTACHMENTS = 'attachments';

  constructor(
    formBuilder: UntypedFormBuilder,
    private postfachService: PostfachService,
  ) {
    super(formBuilder);
  }

  protected initForm(): UntypedFormGroup {
    return this.formBuilder.group({
      [PostfachMailFormservice.FIELD_SUBJECT]: new UntypedFormControl(),
      [PostfachMailFormservice.FIELD_MAIL_BODY]: new UntypedFormControl(),
      [PostfachMailFormservice.FIELD_REPLY_OPTION]: new UntypedFormControl(ReplyOption.POSSIBLE),
      [PostfachMailFormservice.FIELD_ATTACHMENTS]: new UntypedFormArray([]),
    });
  }

  protected getPathPrefix(): string {
    return PostfachMailFormservice.FIELD_PATH_PREFIX;
  }

  protected doSubmit(): Observable<StateResource<CommandResource>> {
    if (this.isPatch()) {
      return this.postfachService.sendExistingMail(this.source, this.getFormValue());
    } else {
      return this.postfachService.sendMail(this.getFormValue());
    }
  }
}
