/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { HttpErrorResponse, HttpStatusCode } from '@angular/common/http';
import { faker } from '@faker-js/faker';
import { ApiError, InvalidParam, Issue, IssueParam, ProblemDetail } from '../src/lib/tech.model';
import { ValidationMessageCode } from '../src/lib/validation/tech.validation.messages';

export function createIssueParam(): IssueParam {
  return {
    name: faker.random.word(),
    value: faker.random.word(),
  };
}

export function createIssue(): Issue {
  return {
    field: faker.random.word(),
    message: faker.random.words(10),
    messageCode: faker.random.words(5),
    parameters: [createIssueParam()],
  };
}

export function createApiError(issues: Issue[] = [createIssue()]): ApiError {
  return { issues };
}

export function createHttpErrorResponse(apiError: ApiError = null): HttpErrorResponse {
  return <HttpErrorResponse>{
    error: {
      error: apiError ?? createApiError(),
    },
  };
}

export function createProblemDetail(
  invalidParams: InvalidParam[] = [createInvalidParam()],
): ProblemDetail {
  return {
    status: HttpStatusCode.UnprocessableEntity,
    title: faker.random.word(),
    type: faker.random.word(),
    instance: faker.internet.url(),
    detail: faker.random.word(),
    'invalid-params': invalidParams,
  };
}

export function createInvalidParam(): InvalidParam {
  return { name: faker.random.word(), reason: ValidationMessageCode.VALIDATION_FIELD_EMPTY };
}
