import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatIcon } from '@angular/material/icon';
import { RouterTestingModule } from '@angular/router/testing';
import faker from '@faker-js/faker';
import { getElementFromFixture } from '@alfa-client/test-utils';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MatTooltipDirective } from 'libs/ui/src/lib/ui/mattooltip/mattooltip.directive';
import { BackButtonComponent } from './back-button.component';

describe('BackButtonComponent', () => {
  let component: BackButtonComponent;
  let fixture: ComponentFixture<BackButtonComponent>;

  const backButton: string = getDataTestIdOf('back-button');
  const linkToAsArray: string[] = [faker.word.adverb(), faker.word.adverb()];
  const linkToAsString: string = '/' + linkToAsArray.join('/');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [BackButtonComponent, MatIcon, MatTooltipDirective],
      imports: [RouterTestingModule],
    }).compileComponents();

    fixture = TestBed.createComponent(BackButtonComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('label', () => {
    beforeEach(() => {
      component.label = faker.lorem.sentence();
      fixture.detectChanges();
    });

    it('should set aria-label attribute', () => {
      const backButtonElement: HTMLAnchorElement = getElementFromFixture(fixture, backButton);

      expect(backButtonElement).toHaveAttribute('aria-label', component.label);
    });

    it('should set title attribute', () => {
      const backButtonElement: HTMLAnchorElement = getElementFromFixture(fixture, backButton);

      expect(backButtonElement).toHaveAttribute('title', component.label);
    });
  });

  describe('linkTo', () => {
    it('string path should set href attribute', () => {
      component.linkTo = linkToAsString;
      fixture.detectChanges();

      const backButtonElement: HTMLAnchorElement = getElementFromFixture(fixture, backButton);

      expect(backButtonElement).toHaveAttribute('href', component.linkTo);
    });

    it('empty string path should set href attribute to base url', () => {
      component.linkTo = '';
      fixture.detectChanges();

      const backButtonElement: HTMLAnchorElement = getElementFromFixture(fixture, backButton);

      expect(backButtonElement).toHaveAttribute('href', '/');
    });

    it('array path should set href attribute', () => {
      component.linkTo = linkToAsArray;
      fixture.detectChanges();

      const backButtonElement: HTMLAnchorElement = getElementFromFixture(fixture, backButton);

      expect(backButtonElement).toHaveAttribute('href', linkToAsString);
    });

    it('empty array path should href attribute to base url', () => {
      component.linkTo = [];
      fixture.detectChanges();

      const backButtonElement: HTMLAnchorElement = getElementFromFixture(fixture, backButton);

      expect(backButtonElement).toHaveAttribute('href', '/');
    });
  });
});
