import { CommandResource } from '@alfa-client/command-shared';
import { AbstractFormService, StateResource } from '@alfa-client/tech-shared';
import { VorgangService, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { Injectable } from '@angular/core';
import { UntypedFormBuilder, UntypedFormControl, UntypedFormGroup } from '@angular/forms';
import { isEmpty } from 'lodash-es';
import { Observable } from 'rxjs';

@Injectable()
export class AktenzeichenEditDialogFormservice extends AbstractFormService<CommandResource> {
  static readonly AKTENZEICHEN_FIELD = 'aktenzeichen';

  private vorgang: VorgangWithEingangResource;

  constructor(
    formBuilder: UntypedFormBuilder,
    private readonly vorgangService: VorgangService,
  ) {
    super(formBuilder);
  }

  protected initForm(): UntypedFormGroup {
    return this.formBuilder.group({
      [AktenzeichenEditDialogFormservice.AKTENZEICHEN_FIELD]: new UntypedFormControl(null),
    });
  }

  protected doSubmit(): Observable<StateResource<CommandResource>> {
    return this.vorgangService.setAktenzeichen(this.vorgang, this.getValue());
  }

  protected getPathPrefix(): string {
    return AktenzeichenEditDialogFormservice.AKTENZEICHEN_FIELD;
  }

  getAktenzeichenFormControl(): UntypedFormControl {
    return this.getFormControl();
  }

  public getValue(): string {
    const value: string = this.getAktenzeichenFormControl().value;
    return isEmpty(value) ? null : value.trim();
  }

  public setVorgangWithEingangResource(vorgang: VorgangWithEingangResource): void {
    this.vorgang = vorgang;
  }

  public setAktenzeichen(aktenzeichen: string): void {
    this.getFormControl().setValue(aktenzeichen);
  }

  public valueChanges(): Observable<any> {
    return this.getFormControl().valueChanges;
  }

  private getFormControl(): UntypedFormControl {
    return <UntypedFormControl>(
      this.form.controls[AktenzeichenEditDialogFormservice.AKTENZEICHEN_FIELD]
    );
  }
}
