/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { VerticalBinaryFileListComponent } from '@alfa-client/binary-file';
import { mock } from '@alfa-client/test-utils';
import {
  VorgangService,
  VorgangWithEingangLinkRel,
  VorgangWithEingangResource,
} from '@alfa-client/vorgang-shared';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { VorgangDetailAttachmentListComponent } from './vorgang-detail-attachment-list.component';

describe('VorgangDetailAttachmentListComponent', () => {
  let component: VorgangDetailAttachmentListComponent;
  let fixture: ComponentFixture<VorgangDetailAttachmentListComponent>;

  const vorgangService = mock(VorgangService);
  const vorgangWithAttachments: VorgangWithEingangResource = createVorgangWithEingangResource([
    VorgangWithEingangLinkRel.ATTACHMENTS,
  ]);

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        VorgangDetailAttachmentListComponent,
        MockComponent(VerticalBinaryFileListComponent),
      ],
      providers: [
        {
          provide: VorgangService,
          useValue: vorgangService,
        },
      ],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(VorgangDetailAttachmentListComponent);
    component = fixture.componentInstance;
    component.vorgangWithEingang = createVorgangWithEingangResource();
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('on vorgang with "attachment" link', () => {
    it('should call vorgangService methods', () => {
      component.vorgangWithEingang = vorgangWithAttachments;

      component.ngOnInit();

      expect(vorgangService.getAttachments).toHaveBeenCalled();
    });
  });
});
