import { BescheidLinkRel, BescheidResource, BescheidService } from '@alfa-client/bescheid-shared';
import { BinaryFile2ContainerComponent } from '@alfa-client/binary-file';
import { createStateResource } from '@alfa-client/tech-shared';
import { Mock, existsAsHtmlElement, mock, notExistsAsHtmlElement } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { getUrl } from '@ngxp/rest';
import { AttachmentContainerComponent } from '@ods/system';
import { createBescheidResource } from 'libs/bescheid-shared/src/test/bescheid';
import { createBinaryFileResource } from 'libs/binary-file-shared/test/binary-file';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { createApiError } from 'libs/tech-shared/test/error';
import { MockComponent } from 'ng-mocks';
import { VorgangDetailBescheidenFormErrorComponent } from '../vorgang-detail-bescheiden-form-error/vorgang-detail-bescheiden-form-error.component';
import { VorgangDetailBescheidenResultDokumentComponent } from './vorgang-detail-bescheiden-result-dokument.component';

describe('VorgangDetailBescheidenResultDokumentComponent', () => {
  let component: VorgangDetailBescheidenResultDokumentComponent;
  let fixture: ComponentFixture<VorgangDetailBescheidenResultDokumentComponent>;

  const createBescheidDocumentError: string = getDataTestIdOf('create-bescheid-document-error');
  const uploadBescheidDocumentError: string = getDataTestIdOf('upload-bescheid-document-error');
  const missingBescheidDocumentErrorMessage: string = getDataTestIdOf(
    'missing-bescheid-document-error-message',
  );

  let bescheidService: Mock<BescheidService>;

  beforeEach(async () => {
    bescheidService = mock(BescheidService);

    await TestBed.configureTestingModule({
      declarations: [
        VorgangDetailBescheidenResultDokumentComponent,
        MockComponent(BinaryFile2ContainerComponent),
        MockComponent(VorgangDetailBescheidenFormErrorComponent),
        MockComponent(AttachmentContainerComponent),
      ],
      providers: [
        {
          provide: BescheidService,
          useValue: bescheidService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(VorgangDetailBescheidenResultDokumentComponent);
    component = fixture.componentInstance;
    component.bescheidDocumentFile = createStateResource(createBinaryFileResource());
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('render', () => {
    it('should show error message for missing bescheid document', () => {
      component.showMissingBescheidDocumentError = true;

      fixture.detectChanges();

      existsAsHtmlElement(fixture, missingBescheidDocumentErrorMessage);
    });

    it('should not show error message for missing bescheid document', () => {
      component.showMissingBescheidDocumentError = false;

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, missingBescheidDocumentErrorMessage);
    });
  });

  describe('handle bescheid document', () => {
    it('should call service to load bescheid document by uri if link is present', () => {
      const bescheid: BescheidResource = createBescheidResource([
        BescheidLinkRel.BESCHEID_DOCUMENT,
      ]);

      component.handleBescheidDocument(bescheid);

      expect(bescheidService.loadBescheidDocumentByUri).toHaveBeenCalledWith(
        getUrl(bescheid, BescheidLinkRel.BESCHEID_DOCUMENT),
      );
    });

    it('should NOT call service if link is missing', () => {
      component.handleBescheidDocument(createBescheidResource());

      expect(bescheidService.loadBescheidDocumentByUri).not.toHaveBeenCalled();
    });
  });

  describe('upload bescheid document error', () => {
    it('should be shown if error exists', () => {
      component.uploadBescheidDocumentInProgress = { loading: false, error: createApiError() };

      fixture.detectChanges();

      existsAsHtmlElement(fixture, uploadBescheidDocumentError);
    });

    it('should be hidden on non error', () => {
      component.uploadBescheidDocumentInProgress = { loading: false };

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, uploadBescheidDocumentError);
    });
  });

  describe('create bescheid document error', () => {
    it('should be shown if error exists', () => {
      component.createBescheidDocumentInProgress = { loading: false, error: createApiError() };

      fixture.detectChanges();

      existsAsHtmlElement(fixture, createBescheidDocumentError);
    });

    it('should be hidden on non error', () => {
      component.createBescheidDocumentInProgress = { loading: false };

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, createBescheidDocumentError);
    });
  });
});
