import { BescheidLinkRel, BescheidResource, BescheidService } from '@alfa-client/bescheid-shared';
import { CommandOrder, CommandResource } from '@alfa-client/command-shared';
import { StateResource, createStateResource } from '@alfa-client/tech-shared';
import {
  Mock,
  dispatchEventFromFixture,
  existsAsHtmlElement,
  getElementFromFixture,
  mock,
  notExistsAsHtmlElement,
} from '@alfa-client/test-utils';
import { OzgcloudButtonWithSpinnerComponent } from '@alfa-client/ui';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ButtonWithSpinnerComponent } from '@ods/component';
import {
  createBescheidResource,
  createBescheidStateResource,
} from 'libs/bescheid-shared/src/test/bescheid';
import { CommandLinkRel } from 'libs/command-shared/src/lib/command.linkrel';
import { createCommandResource } from 'libs/command-shared/test/command';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { BehaviorSubject, EMPTY, first, of } from 'rxjs';
import { singleColdCompleted } from '../../../../../../tech-shared/test/marbles';
import { BescheidenFormService } from '../bescheiden.formservice';
import { VorgangDetailBescheidenResultAttachmentsComponent } from './vorgang-detail-bescheiden-result-attachments/vorgang-detail-bescheiden-result-attachments.component';
import { VorgangDetailBescheidenResultDokumentComponent } from './vorgang-detail-bescheiden-result-dokument/vorgang-detail-bescheiden-result-dokument.component';
import { VorgangDetailBescheidenResultNachrichtComponent } from './vorgang-detail-bescheiden-result-nachricht/vorgang-detail-bescheiden-result-nachricht.component';
import { VorgangDetailBescheidenResultStatusComponent } from './vorgang-detail-bescheiden-result-status/vorgang-detail-bescheiden-result-status.component';
import { VorgangDetailBescheidenResultComponent } from './vorgang-detail-bescheiden-result.component';

describe('VorgangDetailBescheidenResultComponent', () => {
  let component: VorgangDetailBescheidenResultComponent;
  let fixture: ComponentFixture<VorgangDetailBescheidenResultComponent>;

  let bescheidService: Mock<BescheidService>;
  let formService: Mock<BescheidenFormService>;

  const bescheidStatus: string = getDataTestIdOf('bescheid-status');
  const bescheidDocument: string = getDataTestIdOf('bescheid-document');
  const bescheidAttachments: string = getDataTestIdOf('bescheid-attachments');
  const saveAndSendButton: string = getDataTestIdOf('confirm-and-save-button');
  const sendButton: string = getDataTestIdOf('send-button');
  const nachrichtAntragstellerComponent = getDataTestIdOf('bescheid-nachricht-an-antragsteller');

  beforeEach(async () => {
    bescheidService = mock(BescheidService);
    bescheidService.getBescheidDraft.mockReturnValue(EMPTY);

    formService = mock(BescheidenFormService);
    formService.getBescheidChanges.mockReturnValue(
      new BehaviorSubject({ beschiedenAm: new Date(), bewilligt: false }),
    );
    await TestBed.configureTestingModule({
      declarations: [
        VorgangDetailBescheidenResultComponent,
        MockComponent(VorgangDetailBescheidenResultStatusComponent),
        MockComponent(VorgangDetailBescheidenResultDokumentComponent),
        MockComponent(VorgangDetailBescheidenResultAttachmentsComponent),
        MockComponent(OzgcloudButtonWithSpinnerComponent),
        MockComponent(VorgangDetailBescheidenResultNachrichtComponent),
        MockComponent(ButtonWithSpinnerComponent),
      ],
      providers: [
        {
          provide: BescheidService,
          useValue: bescheidService,
        },
        {
          provide: BescheidenFormService,
          useValue: formService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(VorgangDetailBescheidenResultComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('render Nachricht component', () => {
    it('should render', () => {
      formService.getActiveStep.mockReturnValue(of(3));
      formService.isSendByManual.mockReturnValue(of(false));
      component.ngOnInit();
      component.canSend$ = of(true);

      fixture.detectChanges();

      const element = getElementFromFixture(fixture, nachrichtAntragstellerComponent);
      expect(element).toBeInstanceOf(HTMLElement);
    });

    it.each([1, 2])('should not render in step %d', (step) => {
      formService.getActiveStep.mockReturnValue(of(step));
      formService.isSendByManual.mockReturnValue(of(false));
      component.ngOnInit();
      component.canSend$ = of(true);

      fixture.detectChanges();

      const element = getElementFromFixture(fixture, nachrichtAntragstellerComponent);
      expect(element).not.toBeInstanceOf(HTMLElement);
    });

    it('should not render if send by manual', () => {
      formService.getActiveStep.mockReturnValue(of(3));
      formService.isSendByManual.mockReturnValue(of(true));
      component.ngOnInit();
      component.canSend$ = of(true);

      fixture.detectChanges();

      const element = getElementFromFixture(fixture, nachrichtAntragstellerComponent);
      expect(element).not.toBeInstanceOf(HTMLElement);
    });

    it('should not render if postfach not configured', () => {
      formService.getActiveStep.mockReturnValue(of(3));
      formService.isSendByManual.mockReturnValue(of(false));
      component.ngOnInit();
      component.canSend$ = of(false);

      fixture.detectChanges();

      const element = getElementFromFixture(fixture, nachrichtAntragstellerComponent);
      expect(element).not.toBeInstanceOf(HTMLElement);
    });
  });

  describe('render send button', () => {
    it('should render', () => {
      formService.getActiveStep.mockReturnValue(of(3));
      formService.isSendByManual.mockReturnValue(of(false));
      bescheidService.getBescheidDraft.mockReturnValue(
        of(createStateResource(createBescheidResource())),
      );
      component.ngOnInit();
      component.canSend$ = of(true);

      fixture.detectChanges();

      const element = getElementFromFixture(fixture, sendButton);
      expect(element).toBeInstanceOf(HTMLElement);
    });

    it.each([1, 2])('should not render in step %d', (step) => {
      formService.getActiveStep.mockReturnValue(of(step));
      formService.isSendByManual.mockReturnValue(of(false));
      component.ngOnInit();
      component.canSend$ = of(true);

      fixture.detectChanges();

      const element = getElementFromFixture(fixture, sendButton);
      expect(element).not.toBeInstanceOf(HTMLElement);
    });

    it('should not render if send by manual', () => {
      formService.getActiveStep.mockReturnValue(of(3));
      formService.isSendByManual.mockReturnValue(of(true));
      component.ngOnInit();
      component.canSend$ = of(true);

      fixture.detectChanges();

      const element = getElementFromFixture(fixture, sendButton);
      expect(element).not.toBeInstanceOf(HTMLElement);
    });

    it('should not render if postfach not configured', () => {
      formService.getActiveStep.mockReturnValue(of(3));
      formService.isSendByManual.mockReturnValue(of(false));
      component.ngOnInit();
      component.canSend$ = of(false);

      fixture.detectChanges();

      const element = getElementFromFixture(fixture, sendButton);
      expect(element).not.toBeInstanceOf(HTMLElement);
    });
  });

  describe('ngOnInit', () => {
    it('should call service to get bescheid document file', () => {
      component.ngOnInit();

      expect(bescheidService.getBescheidDocumentFile).toHaveBeenCalled();
    });

    it('should call service to get bescheid draft', () => {
      component.ngOnInit();

      expect(bescheidService.getBescheidDraft).toHaveBeenCalled();
    });

    it('should call service to get bescheid document', () => {
      component.ngOnInit();

      expect(bescheidService.getBescheidDocument).toHaveBeenCalled();
    });

    it('should call formservice to get active step', () => {
      component.ngOnInit();

      expect(formService.getActiveStep).toHaveBeenCalled();
    });

    it('should call formservice to get current bescheid/formular', () => {
      component.ngOnInit();

      expect(formService.getBescheidChanges).toHaveBeenCalled();
    });

    it('should call formservice to get sendByManual', () => {
      component.ngOnInit();

      expect(formService.isSendByManual).toHaveBeenCalled();
    });

    it('should call service to get upload bescheid document in progress', () => {
      component.ngOnInit();

      expect(bescheidService.getUploadBescheidDocumentInProgress).toHaveBeenCalled();
    });

    it('should call service to get create bescheid document in progress', () => {
      component.ngOnInit();

      expect(bescheidService.getCreateBescheidDocumentInProgress).toHaveBeenCalled();
    });

    describe('canSave$', () => {
      it('should emit true', () => {
        bescheidService.getBescheidDraft.mockReturnValue(
          of(createBescheidStateResource([BescheidLinkRel.BESCHEIDEN])),
        );
        component.ngOnInit();

        expect(component.canSave$).toBeObservable(singleColdCompleted(true));
      });

      it('should emit false', () => {
        bescheidService.getBescheidDraft.mockReturnValue(of(createBescheidStateResource()));
        component.ngOnInit();

        expect(component.canSave$).toBeObservable(singleColdCompleted(false));
      });
    });

    describe('canSend$', () => {
      it('should emit true', () => {
        bescheidService.getBescheidDraft.mockReturnValue(
          of(createBescheidStateResource([BescheidLinkRel.BESCHEIDEN_UND_SENDEN])),
        );
        component.ngOnInit();

        expect(component.canSend$).toBeObservable(singleColdCompleted(true));
      });

      it('should emit false', () => {
        bescheidService.getBescheidDraft.mockReturnValue(of(createBescheidStateResource()));
        component.ngOnInit();

        expect(component.canSend$).toBeObservable(singleColdCompleted(false));
      });
    });

    describe('showMissingBescheidDocumentError$', () => {
      it('should emit true', () => {
        formService.getShowMissingBescheidDocumentError.mockReturnValue(of(true));

        component.ngOnInit();

        expect(component.showMissingBescheidDocumentError$).toBeObservable(
          singleColdCompleted(true),
        );
      });

      it('should emit false', () => {
        formService.getShowMissingBescheidDocumentError.mockReturnValue(of(false));

        component.ngOnInit();

        expect(component.showMissingBescheidDocumentError$).toBeObservable(
          singleColdCompleted(false),
        );
      });
    });
  });

  describe('deleteBescheidDocument', () => {
    it('should call service to delete bescheid document', () => {
      component.deleteBescheidDocument();

      expect(bescheidService.deleteBescheidDocument).toHaveBeenCalled();
    });

    it('should clear bescheid document file', () => {
      component.deleteBescheidDocument();

      expect(formService.clearBescheidDocumentFile).toHaveBeenCalled();
    });
  });

  describe('save and send manually', () => {
    const bescheidDraft: BescheidResource = createBescheidResource();
    const bescheidStateResource: StateResource<BescheidResource> =
      createStateResource(bescheidDraft);

    beforeEach(() => {
      component.bescheidDraftStateResource$ = of(bescheidStateResource);
      component.sendByManual$ = of(true);
      component.activeStep$ = of(3);
    });

    it('should clear nachricht', () => {
      component.doUpdateAndSend = jest.fn();

      component.saveAndSendManually(bescheidDraft);

      expect(formService.clearNachricht).toHaveBeenCalled();
    });

    it('should call component on event dispatch', () => {
      component.saveAndSendManually = jest.fn();
      component.canSave$ = of(true);

      fixture.detectChanges();

      dispatchEventFromFixture(fixture, saveAndSendButton, 'clickEmitter');

      expect(component.saveAndSendManually).toHaveBeenCalledWith(bescheidDraft);
    });

    it('should call do update and send', () => {
      component.doUpdateAndSend = jest.fn();

      component.saveAndSendManually(bescheidDraft);

      expect(component.doUpdateAndSend).toHaveBeenCalled();
    });
  });

  describe('save and send with Nachricht', () => {
    const bescheidDraft: BescheidResource = createBescheidResource();
    const bescheidStateResource: StateResource<BescheidResource> =
      createStateResource(bescheidDraft);

    beforeEach(() => {
      component.bescheidDraftStateResource$ = of(bescheidStateResource);
      component.sendByManual$ = of(true);
      component.activeStep$ = of(3);
    });

    it('should call do update and send', () => {
      component.doUpdateAndSend = jest.fn();

      component.saveAndSendWithNachricht(bescheidDraft);

      expect(component.doUpdateAndSend).toHaveBeenCalled();
    });
  });

  describe('doUpdateAndSend', () => {
    const bescheidDraft: BescheidResource = createBescheidResource();
    const commandStateResource: StateResource<CommandResource> = createStateResource(
      createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]),
    );

    beforeEach(() => {
      component.sendByManual$ = of(true);
      formService.submit = jest.fn().mockReturnValue(of(commandStateResource));
    });

    it('should call formservice submit', () => {
      component.doUpdateAndSend(bescheidDraft, jest.fn()).pipe(first()).subscribe();

      expect(formService.submit).toHaveBeenCalled();
    });

    it('should call send on successfully done command', () => {
      const sendMock = jest.fn();

      component.doUpdateAndSend(bescheidDraft, sendMock).pipe(first()).subscribe();

      expect(sendMock).toHaveBeenCalled();
    });

    it('should close dialog on successfully done command', () => {
      const sendMock = jest.fn().mockReturnValue(
        of(
          createStateResource({
            ...createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]),
            order: CommandOrder.SEND_BESCHEID,
          }),
        ),
      );
      const emit = (component.closeDialog.emit = jest.fn());

      component.doUpdateAndSend(bescheidDraft, sendMock).subscribe();

      expect(emit).toHaveBeenCalled();
    });
  });

  describe('bescheid status text', () => {
    it('should NOT be visible on NOT send by manual', () => {
      component.sendByManual$ = of(false);

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, bescheidStatus);
    });

    it('should NOT be visible on step 3 and NOT send by manual', () => {
      component.sendByManual$ = of(false);
      component.activeStep$ = of(3);

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, bescheidStatus);
    });
  });

  describe('bescheid document', () => {
    it('should NOT be visible on NOT send by manual', () => {
      component.sendByManual$ = of(false);

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, bescheidDocument);
    });

    it('should NOT be visible on step 3', () => {
      component.sendByManual$ = of(false);
      component.activeStep$ = of(3);

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, bescheidDocument);
    });

    it('should NOT be visible on step 1', () => {
      component.sendByManual$ = of(false);
      component.activeStep$ = of(1);

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, bescheidDocument);
    });

    it('should be visible in step 3 when link exists', () => {
      component.sendByManual$ = of(false);
      component.activeStep$ = of(3);
      component.canSend$ = of(true);

      fixture.detectChanges();

      existsAsHtmlElement(fixture, bescheidDocument);
    });
  });

  describe('bescheid attachments', () => {
    it('should NOT be visible on NOT send by manual', () => {
      component.sendByManual$ = of(false);

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, bescheidAttachments);
    });

    it('should NOT be visible on step 3', () => {
      component.sendByManual$ = of(false);
      component.activeStep$ = of(3);

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, bescheidAttachments);
    });

    it('should NOT be visible on step 1', () => {
      component.sendByManual$ = of(false);
      component.activeStep$ = of(1);

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, bescheidAttachments);
    });

    it('should be visible in step 3 when postfach is configured', () => {
      component.sendByManual$ = of(false);
      component.activeStep$ = of(3);
      component.canSend$ = of(true);

      fixture.detectChanges();

      existsAsHtmlElement(fixture, bescheidAttachments);
    });
  });

  describe('update and bescheiden', () => {
    it('should NOT be visible on step 1', () => {
      component.sendByManual$ = of(false);

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, saveAndSendButton);
    });

    it('should NOT be visible on setp 2', () => {
      component.sendByManual$ = of(false);

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, saveAndSendButton);
    });
    it('should NOT be visible on step 3 and NOT send by manual', () => {
      component.sendByManual$ = of(false);

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, saveAndSendButton);
    });
  });
});
