import { BescheidLinkRel, BescheidService } from '@alfa-client/bescheid-shared';
import { HasLinkPipe, createStateResource } from '@alfa-client/tech-shared';
import {
  Mock,
  dispatchEventFromFixture,
  existsAsHtmlElement,
  mock,
  notExistsAsHtmlElement,
} from '@alfa-client/test-utils';
import { OzgcloudButtonWithSpinnerComponent } from '@alfa-client/ui';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import {
  BescheidGenerateIconComponent,
  ButtonCardComponent,
  SpinnerIconComponent,
} from '@ods/system';
import {
  createBescheidResource,
  createUploadFileInProgress,
} from 'libs/bescheid-shared/src/test/bescheid';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { BescheidenFormService } from '../../../../bescheiden.formservice';
import { VorgangDetailBescheidenBescheidAutomatischErstellenComponent } from './vorgang-detail-bescheiden-bescheid-automatisch-erstellen.component';

describe('VorgangDetailBescheidenBescheidAutomatischErstellenComponent', () => {
  let component: VorgangDetailBescheidenBescheidAutomatischErstellenComponent;
  let fixture: ComponentFixture<VorgangDetailBescheidenBescheidAutomatischErstellenComponent>;

  const createBescheidDocumentButton: string = getDataTestIdOf('create-bescheid-document-button');

  let bescheidService: Mock<BescheidService>;
  let formService: Mock<BescheidenFormService>;

  beforeEach(async () => {
    bescheidService = mock(BescheidService);
    formService = mock(BescheidenFormService);

    await TestBed.configureTestingModule({
      imports: [HasLinkPipe],
      declarations: [
        VorgangDetailBescheidenBescheidAutomatischErstellenComponent,
        MockComponent(OzgcloudButtonWithSpinnerComponent),
        MockComponent(ButtonCardComponent),
        MockComponent(SpinnerIconComponent),
        MockComponent(BescheidGenerateIconComponent),
      ],
      providers: [
        {
          provide: BescheidService,
          useValue: bescheidService,
        },
        {
          provide: BescheidenFormService,
          useValue: formService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(VorgangDetailBescheidenBescheidAutomatischErstellenComponent);
    component = fixture.componentInstance;
    component.bescheidDraftStateResource = createStateResource(createBescheidResource());
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('create bescheid document button', () => {
    beforeEach(() => {
      component.bescheidDraftStateResource = createStateResource(
        createBescheidResource([BescheidLinkRel.CREATE_DOCUMENT]),
      );

      fixture.detectChanges();
    });
    it('should call bescheid service on click', () => {
      dispatchEventFromFixture(fixture, createBescheidDocumentButton, 'click');

      expect(bescheidService.createBescheidDocument).toHaveBeenCalled();
    });

    it('should be hidden if link is NOT present', () => {
      component.bescheidDraftStateResource = createStateResource(createBescheidResource());

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, createBescheidDocumentButton);
    });

    it('should be visible if link is present', () => {
      component.bescheidDraftStateResource = createStateResource(
        createBescheidResource([BescheidLinkRel.CREATE_DOCUMENT]),
      );
      fixture.detectChanges();

      existsAsHtmlElement(fixture, createBescheidDocumentButton);
    });

    it('should clear nachricht', () => {
      bescheidService.createBescheidDocument.mockReturnValue(
        of({ ...createUploadFileInProgress(), loading: false, error: null }),
      );

      component.createBescheidDocument();

      component.createBescheidDocumentInProgress$.subscribe();
      expect(formService.clearNachricht).toHaveBeenCalled();
    });

    it('should not clear nachricht when loading', () => {
      bescheidService.createBescheidDocument.mockReturnValue(
        of({ ...createUploadFileInProgress(), error: null }),
      );

      component.createBescheidDocument();

      component.createBescheidDocumentInProgress$.subscribe();
      expect(formService.clearNachricht).not.toHaveBeenCalled();
    });

    it('should not clear nachricht when error', () => {
      bescheidService.createBescheidDocument.mockReturnValue(
        of({ ...createUploadFileInProgress(), loading: false }),
      );

      component.createBescheidDocument();

      component.createBescheidDocumentInProgress$.subscribe();
      expect(formService.clearNachricht).not.toHaveBeenCalled();
    });
  });
});
