import { HttpError, StateResource, hasError, isLoaded } from '@alfa-client/tech-shared';
import {
  VorgangService,
  VorgangWithEingangLinkRel,
  VorgangWithEingangResource,
} from '@alfa-client/vorgang-shared';
import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { Resource, hasLink } from '@ngxp/rest';
import { BehaviorSubject, Observable, combineLatest, filter, first, map } from 'rxjs';
import { BescheidenFormService } from '../../bescheiden.formservice';

@Component({
  selector: 'alfa-vorgang-detail-bescheiden-steps-content',
  templateUrl: './vorgang-detail-bescheiden-steps-content.component.html',
  styles: [':host {@apply flex flex-col}'],
})
export class VorgangDetailBescheidenStepsContentComponent implements OnInit {
  readonly activeStep$ = new BehaviorSubject(1);

  @Input() set activeStep(step: number) {
    this.activeStep$.next(step);
  }
  @Output() activeStepChange = new EventEmitter<number>();

  showBescheidErstellungUeberspringen$: Observable<boolean>;

  constructor(
    private readonly vorgangService: VorgangService,
    private formService: BescheidenFormService,
  ) {}

  ngOnInit(): void {
    this.showBescheidErstellungUeberspringen$ = combineLatest([
      this.vorgangService.getVorgangWithEingang().pipe(
        filter(isLoaded),
        map((stateResource: StateResource<VorgangWithEingangResource>) =>
          hasLink(stateResource.resource, VorgangWithEingangLinkRel.ABSCHLIESSEN),
        ),
      ),
      this.activeStep$.pipe(map((step) => step === 1)),
    ]).pipe(map(([hasAbschliessenLink, isFirstStep]) => hasAbschliessenLink && isFirstStep));
  }

  public changeActiveStep(step: number): void {
    if (this.canChangeTo(step)) {
      this.formService
        .submit()
        .pipe(filter(this.noError), first())
        .subscribe(() => {
          this.formService.setActiveStep(step);
          this.activeStepChange.emit(step);
        });
    }
  }

  canChangeTo(nextStep: number): boolean {
    if (nextStep < 3) {
      return true;
    }
    return this.formService.validateBescheidDocumentExists();
  }

  noError(stateResource: StateResource<Resource | HttpError>): boolean {
    return isLoaded(stateResource) && !hasError(stateResource);
  }
}
