/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import {
  AbstractFormService,
  StateResource,
  createFutureDate,
  formatForDatabase,
} from '@alfa-client/tech-shared';
import {
  Wiedervorlage,
  WiedervorlageResource,
  WiedervorlageService,
} from '@alfa-client/wiedervorlage-shared';
import { Injectable } from '@angular/core';
import {
  UntypedFormArray,
  UntypedFormBuilder,
  UntypedFormControl,
  UntypedFormGroup,
} from '@angular/forms';
import { Observable } from 'rxjs';

@Injectable()
export class WiedervorlageFormService extends AbstractFormService<WiedervorlageResource> {
  static readonly FIELD_BETREFF = 'betreff';
  static readonly FIELD_BESCHREIBUNG = 'beschreibung';
  static readonly FIELD_FRIST = 'frist';
  static readonly FIELD_ATTACHMENTS = 'attachments';

  static readonly FIELD_PATH_PREFIX = 'wiedervorlage';

  constructor(
    formBuilder: UntypedFormBuilder,
    private wiedervorlageService: WiedervorlageService,
  ) {
    super(formBuilder);
  }

  protected initForm(): UntypedFormGroup {
    return this.formBuilder.group({
      [WiedervorlageFormService.FIELD_BETREFF]: new UntypedFormControl(null),
      [WiedervorlageFormService.FIELD_FRIST]: new UntypedFormControl(createFutureDate(14)),
      [WiedervorlageFormService.FIELD_BESCHREIBUNG]: new UntypedFormControl(),
      [WiedervorlageFormService.FIELD_ATTACHMENTS]: new UntypedFormArray([]),
    });
  }

  protected getPathPrefix(): string {
    return WiedervorlageFormService.FIELD_PATH_PREFIX;
  }

  protected doSubmit(): Observable<StateResource<any>> {
    if (this.isPatch()) {
      return this.wiedervorlageService.saveWiedervorlage(this.getSourceValue(), this.getValue());
    } else {
      return this.wiedervorlageService.createWiedervorlage(this.getValue());
    }
  }

  getValue(): Wiedervorlage {
    const formValue = super.getFormValue();
    return { ...formValue, frist: formatForDatabase(formValue.frist) };
  }
}
