package de.ozgcloud.alfa.bescheid;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.alfa.common.binaryfile.FileIdMapper;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;

class BescheidMapperTest {

	@Mock
	private FileIdMapper fileIdMapper;
	@InjectMocks
	private final BescheidMapper mapper = Mappers.getMapper(BescheidMapper.class);

	@Nested
	class TestFromGrpc {

		@Test
		void shouldMapId() {
			var bescheid = map();

			assertThat(bescheid.getId()).isEqualTo(GrpcBescheidTestFactory.ID);
		}

		@Test
		void shouldMapVorgangId() {
			var bescheid = map();

			assertThat(bescheid.getVorgangId()).isEqualTo(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldMapBewilligt() {
			var bescheid = map();

			assertThat(bescheid.getBewilligt()).isEqualTo(GrpcBescheidTestFactory.BEWILLIGT);
		}

		@Test
		void shouldMapBeschiedenAm() {
			var bescheid = map();

			assertThat(bescheid.getBeschiedenAm()).isEqualTo(GrpcBescheidTestFactory.BESCHIEDEN_AM);
		}

		@Test
		void shouldMapBescheidDocument() {
			var bescheid = map();

			assertThat(bescheid.getBescheidDocument()).isEqualTo(GrpcBescheidTestFactory.BESCHEID_DOCUMENT);
		}

		@Test
		void shouldMapAttachments() {
			when(fileIdMapper.toFileId(any())).thenCallRealMethod();

			var bescheid = map();

			assertThat(bescheid.getAttachments()).containsExactlyInAnyOrderElementsOf(BescheidTestFactory.ATTACHMENTS);
		}

		@Test
		void shouldMapNachrichtText() {
			var bescheid = map();

			assertThat(bescheid.getNachrichtText()).isEqualTo(GrpcBescheidTestFactory.NACHRICHT_TEXT);
		}

		@Test
		void shouldMapNachrichtSubject() {
			var bescheid = map();

			assertThat(bescheid.getNachrichtSubject()).isEqualTo(GrpcBescheidTestFactory.NACHRICHT_SUBJECT);
		}

		@Test
		void shouldMapStatus() {
			var bescheid = map();

			assertThat(bescheid.getStatus()).isEqualTo(BescheidTestFactory.STATUS);
		}

		private Bescheid map() {
			return mapper.fromGrpc(GrpcBescheidTestFactory.create(), VorgangHeaderTestFactory.ID);
		}
	}
}
