package de.ozgcloud.alfa.export;

import static org.assertj.core.api.AssertionsForClassTypes.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.TestPropertySource;

public class XdomeaPropertiesTest {

	@Nested
	@SpringBootTest(properties = {"ozgcloud.feature.vorgang-export=false"})
	class OnMissingVorgangExportFeature {

		@Autowired
		XdomeaProperties xdomeaProperties;

		@Test
		void shouldPropertiesBeOfTypeDefault(){
			assertThat(xdomeaProperties).isInstanceOf(DefaultXdomeaProperties.class);
		}

		@Test
		void shouldBehoerdenschluesselThrowException(){
			assertThatThrownBy(() -> xdomeaProperties.getBehoerdenschluessel()).isInstanceOf(UnsupportedOperationException.class);
		}

		@Test
		void shouldBehoerdenschluesselUriThrowException(){
			assertThatThrownBy(() -> xdomeaProperties.getBehoerdenschluesselUri()).isInstanceOf(UnsupportedOperationException.class);
		}

		@Test
		void shouldBehoerdenschluesselVersionThrowException(){
			assertThatThrownBy(() -> xdomeaProperties.getBehoerdenschluesselVersion()).isInstanceOf(UnsupportedOperationException.class);
		}
	}

	@Nested
	@SpringBootTest
	@ActiveProfiles({"itcase"})
	class OnActiveVorgangExportFeature {

		@Autowired
		XdomeaProperties xdomeaProperties;

		@Test
		void shouldPropertiesBeOfTypeConfig(){
			assertThat(xdomeaProperties).isInstanceOf(ExternalXdomeaProperties.class);
		}

		@Test
		void shouldBehoerdenschluesselBeSet(){
			assertThat(xdomeaProperties.getBehoerdenschluessel()).isEqualTo("ABC");
		}

		@Test
		void shouldBehoerdenschluesselUriBeSet(){
			assertThat(xdomeaProperties.getBehoerdenschluesselUri()).isEqualTo("http://meine.behoer.de");
		}

		@Test
		void shouldBehoerdenschluesselVersionBeSet(){
			assertThat(xdomeaProperties.getBehoerdenschluesselVersion()).isEqualTo("1.0.15b");
		}
	}
}
